/**
 * REST demo programs
 */
package demo.rest;

// Pretty Print classes
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

// XML parsing classes
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

// Utility Classes
import java.io.ByteArrayInputStream;
import java.io.InputStream;

import javax.ws.rs.core.MultivaluedMap;
import java.util.Map;
import java.util.Iterator;

/*
 * Static utility methods used by BaseClient, mostly to minimize the
 * import list and keep its code as clean as possible.
 *      
 * @author Eric Armstrong      
 */
public class BaseUtil {
  /*
   * Pretty print an XML stream so elements are indented. 
   * (With thanks to the JavaRanch.)
   * 
   * @param xml_in An XML InputStream
   */
  public static void prettyPrint(InputStream xml_in) {
    try 
    {
      DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
      docFactory.setNamespaceAware(true);
      DocumentBuilder docbuilder = docFactory.newDocumentBuilder();
      Document doc = docbuilder.parse(new InputSource(xml_in));
      
      OutputFormat format = new OutputFormat(doc);
      format.setLineWidth(65);
      format.setIndenting(true);
      format.setIndent(2);
      XMLSerializer serializer = new XMLSerializer(System.out, format);
      serializer.serialize(doc);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }      
  }

  /*
   * Pretty print a MultivaluedMap, where both the keys and the values are 
   * strings, and each key maps to a list of one or more string values. 
   * Lists are displayed in the form:  [item1, item2, ...]
   * 
   * @param map A MultivaluedMap of strings. 
   */
  public static void prettyPrint(MultivaluedMap<String,String> map)
  {  
     System.out.println("Map:");
     Iterator it = map.entrySet().iterator();
     while (it.hasNext()) {
         Map.Entry pairs = (Map.Entry) it.next();
         System.out.println("  " + pairs.getKey() + " = " + pairs.getValue());
     }  
   }  
  
  /*
   * Given an XPath expression, parse the result string for a specific value.
   * @param xpath_expr  An XPATH expression. For example: "/platform/login/sessionId"
   * @param xml The XML string to be parsed.
   */
  public static String xpath_result(String xpath_expr, String xml) 
  {
    try {
      DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
      docFactory.setNamespaceAware(true);
      DocumentBuilder docbuilder = docFactory.newDocumentBuilder();
      InputStream is = new ByteArrayInputStream(xml.getBytes());

      Document doc = docbuilder.parse(new InputSource(is));
      XPathFactory factory = XPathFactory.newInstance();
      XPath xpath = factory.newXPath();
      XPathExpression expr = xpath.compile(xpath_expr);
      Object result = expr.evaluate(doc, XPathConstants.STRING);
      return result.toString();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return null;
  }

  /* Map a filename to the appropriate MIME type, based on its
   * extension. (Inspired by a method in the Apache Wink project.)
   *
   * @param filename Name of the file, with extension, typically
   *                 obtained like this:
   *                    File file = new File(filepath);
   *                    filename = file.getName();
   *                    
   * @return String identifying the MIME type, using same values
   *         as the constants in javax.ws.rs.core.MediaType
   *         (use that for a reference when more are needed)
   */
  public static String mediaType(String filename) {
      String upperFileName = filename.toUpperCase();
      if (upperFileName.endsWith("CSV")) {
          return "text/csv";
      }
      if (upperFileName.endsWith("TSV")) {
          return "text/tab-separated-values";
      }
      if (upperFileName.endsWith("TAB")) {
          return "text/tab-separated-values";
      }
      if (upperFileName.endsWith("HTML")) {
          return "text/html";
      }
      if (upperFileName.endsWith("HTM")) {
          return "text/html";
      }
      if (upperFileName.endsWith("DOC")) {
          return "application/msword";
      }
      if (upperFileName.endsWith("ODS")) {
          return "application/x-vnd.oasis.opendocument.spreadsheet";
      }
      if (upperFileName.endsWith("ODT")) {
          return "application/vnd.oasis.opendocument.text";
      }
      if (upperFileName.endsWith("RTF")) {
          return "application/rtf";
      }
      if (upperFileName.endsWith("SXW")) {
          return "application/vnd.sun.xml.writer";
      }
      if (upperFileName.endsWith("TXT")) {
          return "text/plain";
      }
      if (upperFileName.endsWith("XML")) {
        return "application/xml";
    }
      if (upperFileName.endsWith("XLS")) {
          return "application/vnd.ms-excel";
      }
      if (upperFileName.endsWith("PPT")) {
          return "application/vnd.ms-powerpoint";
      }
      if (upperFileName.endsWith("PPS")) {
          return "application/vnd.ms-powerpoint";
      }
      if (upperFileName.endsWith("JPG")) {
        return "application/octet-stream";
      }
      if (upperFileName.endsWith("GIF")) {
        return "application/octet-stream";
      }
      if (upperFileName.endsWith("PNG")) {
        return "application/octet-stream";
      }
      throw new RuntimeException("Unknown extension.");
  }
}
